package com.persagy.apm.alarmservice.common.configuration;

import javax.servlet.Filter;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.ResourceHandlerRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import com.persagy.apm.common.handler.PoemsContextFilter;
import com.persagy.apm.mybatis.handler.DynamicDataSourceHandler;

/**
 * web拦截器配置
 *
 * @author lixing
 * @version V1.0 2021/5/19 7:42 下午
 */
@Configuration
public class PoemsWebConfigurer implements WebMvcConfigurer {

    @Bean
    public DynamicDataSourceHandler dynamicDataSourceHandler() {
        return new DynamicDataSourceHandler();
    }

    @Bean
    public Filter poemsContextFilter() {
        return new PoemsContextFilter();
    }

    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        // 设置拦截的路径、不拦截的路径、优先级等等
        registry.addInterceptor(dynamicDataSourceHandler()).addPathPatterns("/**");
    }

    @Override
    public void addResourceHandlers(ResourceHandlerRegistry registry) {
        registry.addResourceHandler("doc.html").addResourceLocations("classpath:/META-INF/resources/");
        registry.addResourceHandler("/swagger-resources/**").addResourceLocations("classpath:/META-INF/resources/");
        registry.addResourceHandler("swagger-ui.html").addResourceLocations("classpath:/META-INF/resources/");
        registry.addResourceHandler("/img/**").addResourceLocations("classpath:/META-INF/resources/");
        registry.addResourceHandler("/webjars/**").addResourceLocations("classpath:/META-INF/resources/webjars/");
    }
}
