package com.persagy.apm.alarmservice.dependency.saasweb.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.apm.alarmservice.dependency.saasweb.client.SaasWebClient;
import com.persagy.apm.alarmservice.dependency.saasweb.model.vo.UserProjectsInfoVO;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.vo.ProjectVO;
import com.persagy.apm.common.model.dto.PoemsFeignBaseDTO;
import com.persagy.apm.common.model.vo.SimpleObjVO;
import com.persagy.apm.common.response.PoemsFeignResponse;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * 运维平台feign客户端包装类
 *
 * @author lixing
 * @version V1.0 2021/3/22 2:13 下午
 **/
@Service
public class SaasWebClientWrapper {
    @Autowired
    private SaasWebClient saasWebClient;

    /**
     * 获取用户信息（基本信息和关联的项目）
     *
     * @param poemsFeignBaseDTO 登录信息
     * @return 用户信息
     * @author lixing
     * @version V1.0 2021/5/19 3:36 下午
     */
    public PoemsFeignResponse<UserProjectsInfoVO> getCurrentUserProjectsInfo(PoemsFeignBaseDTO poemsFeignBaseDTO) {
        return saasWebClient.getCurrentUserProjectsInfo(JSON.toJSONString(poemsFeignBaseDTO));
    }

    /**
     * 获取用户信息
     *
     * @param userId 用户id
     * @return 用户信息
     * @author lixing
     * @version V1.0 2021/9/23 2:32 下午
     */
    public SimpleObjVO getUserInfo(String userId) {
        SimpleObjVO result = new SimpleObjVO();
        result.setId(userId);
        PoemsFeignBaseDTO poemsFeignBaseDTO = new PoemsFeignBaseDTO();
        poemsFeignBaseDTO.setDefaultValue();
        poemsFeignBaseDTO.getPuser().setUserId(userId);
        PoemsFeignResponse<UserProjectsInfoVO> currentUserProjectsInfo = getCurrentUserProjectsInfo(poemsFeignBaseDTO);
        if (currentUserProjectsInfo == null) {
            return result;
        }
        List<UserProjectsInfoVO> content = currentUserProjectsInfo.getContent();
        if (content == null) {
            return result;
        }
        UserProjectsInfoVO userProjectsInfoVO = content.get(0);
        if (userProjectsInfoVO == null) {
            return result;
        }
        result.setName(userProjectsInfoVO.getUserName());
        return result;
    }

    /**
     * 查询运维平台所有项目列表
     *
     * @author lixing
     * @version V1.0 2021/12/18 4:21 下午
     */
    public List<ProjectVO> queryProjectList(String groupCode) throws Exception {
        String USERID = "systemId";
        String LOGINDEVICE = "PC";
        JSONObject param = new JSONObject();
        JSONObject puser = new JSONObject();
        puser.put("userId", USERID);
        puser.put("loginDevice", LOGINDEVICE);
        param.put("puser", puser);
        param.put("groupCode",groupCode);
        String resultStr = saasWebClient.queryInUseProjectsByIds(param.toJSONString());
        JSONObject parseObject = JSONObject.parseObject(resultStr);
        if ("failure".equals(parseObject.getString("result"))) {
            throw new Exception("查询分区失败：" + parseObject.getString("reson"));
        }
        List<ProjectVO> result = new ArrayList<>();
        JSONArray jsonArray = parseObject.getJSONArray("content");
        if (jsonArray == null || jsonArray.isEmpty()) {
            return result;
        }

        for (Object object : jsonArray) {
            JSONObject obj = (JSONObject) object;
            ProjectVO projectVO = ProjectVO.builder().
                    groupCode(groupCode).
                    projectId(obj.getString("projectId")).
                    projectName(obj.getString("projectLocalName"))
                    .build();
            result.add(projectVO);
        }
        return result;
    }

}
