package com.persagy.apm.alarmservice.group.alarmrule.controller;

import com.persagy.apm.alarmservice.group.alarmrule.model.dto.AddGroupAlarmRuleDTO;
import com.persagy.apm.alarmservice.group.alarmrule.model.dto.QueryAlarmProjectDTO;
import com.persagy.apm.alarmservice.group.alarmrule.service.IGroupAlarmRuleService;
import com.persagy.apm.alarmservice.project.alarmrule.service.IProjectAlarmRuleService;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.vo.ProjectVO;
import com.persagy.apm.common.response.ApmResponseUpsertVO;
import com.persagy.apm.common.response.CommonResult;
import com.persagy.apm.common.utils.ResultHelper;
import com.persagy.common.enums.ResponseCode;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

@Api(tags = "压力测试接口")
@Validated
@RestController
@RequestMapping("pressure_measurement")
public class PressureMeasurementController {

    @Autowired
    private IGroupAlarmRuleService groupAlarmRuleService;
    @Autowired
    private IProjectAlarmRuleService projectAlarmRuleService;

    @ApiOperation(value = "创建同步项目")
    @PostMapping("/create_alarm_project")
    public void createAlarmProject() {
        try {
            List<ProjectVO> projects = projectAlarmRuleService.getProject();
            // 压测需要的项目总数
            int projectCount = 400;
            if (projects.size() < projectCount){
                StringBuffer sb = new StringBuffer();
                for (int i = projects.size(); i < projectCount; i++) {
                   int num = 1000+i;
                   String projectId = "Pm202112"+num;
                   String projectName = projectId+"广场";
                    sb.append("(");
                    sb.append("'"+num+"',");
                    sb.append("'"+projectId+"',");
                    sb.append("'"+projectName+"',");
                    sb.append("'YL'");
                    sb.append("),");
                }
                String substring = sb.substring(0, sb.length() - 1);
                projectAlarmRuleService.insertProject(substring);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @ApiOperation(value = "创建集团报警规则")
    @PostMapping("/create")
    public CommonResult<ApmResponseUpsertVO> createGroupAlarmRule(
            @Valid @RequestBody AddGroupAlarmRuleDTO addGroupAlarmRuleDTO) {
        String id = null;
        try {
            id = groupAlarmRuleService.createGroupAlarmRulePM(addGroupAlarmRuleDTO);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultHelper.failure(ResponseCode.B0001.getCode(), e.getMessage());
        }
        return ResultHelper.single(new ApmResponseUpsertVO(id));
    }

    @ApiOperation(value = "查询同步项目列表")
    @PostMapping("/query_project")
    public CommonResult<List<ProjectVO>> queryProject(@Valid @RequestBody QueryAlarmProjectDTO queryAlarmProjectDTO) {
        List<ProjectVO> projects = null;
        try {
            projects = projectAlarmRuleService.getProject();
        } catch (Exception e) {
            e.printStackTrace();
            return ResultHelper.failure(ResponseCode.B0001.getCode(), e.getMessage());
        }
        return ResultHelper.single(projects);
    }

}
