package com.persagy.apm.dmpalarm.config;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang.StringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import com.persagy.apm.dmpalarm.criteria.JsonCriteria;

/**
 * @description: 查询切面
 * @author: lixing
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/12/3 9:42 上午
 * @version: V1.0
 */
@Aspect
@Component
public class QueryAspect {
	@Pointcut("execution(public * com.persagy.apm.dmpalarm.service.*.query(..))")
	public void queryPoint() {
	}

	/**
	 * @description: 在查询条件中增加根据项目id和集团编码
	 * @param: joinPoint
	 * @return: void
	 * @exception:
	 * @author: lixing
	 * @company: Persagy Technology Co.,Ltd
	 * @since: 2020/12/3 9:56 上午
	 * @version: V1.0
	 */
	@Before("queryPoint()")
	public void doBefore(JoinPoint joinPoint) {
		ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
		HttpServletRequest request = attributes.getRequest();

		Object[] args = joinPoint.getArgs();
		for (Object arg : args) {
			if (arg instanceof JsonCriteria) {
				JsonCriteria criteria = (JsonCriteria) arg;
				/* 查询接口中，参数为0代表查询所有；报警条目查询例外，0代表查询通用*/
				// 查询参数中不包含groupCode，且param中groupCode不等于0，将groupCode放入查询条件
				if (StringUtils.isEmpty(criteria.getCriteria().getString("groupCode")) &&
						!"0".equals(request.getParameter("groupCode"))) {
					criteria.getCriteria().put("groupCode", request.getParameter("groupCode"));
				}
				// 查询参数中不包含projectId，且param中projectId不等于0，将projectId放入查询条件
				if (StringUtils.isEmpty(criteria.getCriteria().getString("projectId")) &&
						!"0".equals(request.getParameter("projectId"))) {
					criteria.getCriteria().put("projectId", request.getParameter("projectId"));
				}
				// 查询参数中不包含appId，且param中appId不等于0，将appId放入查询条件
				if (StringUtils.isEmpty(criteria.getCriteria().getString("appId")) &&
						!"0".equals(request.getParameter("appId"))) {
					criteria.getCriteria().put("appId", request.getParameter("appId"));
				}
			}
		}
	}
}
