package com.persagy.apm.dmpalarm.config;

import java.util.Iterator;
import java.util.List;

import javax.annotation.Resource;

import org.springframework.amqp.core.AcknowledgeMode;
import org.springframework.amqp.rabbit.config.SimpleRabbitListenerContainerFactory;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.listener.RabbitListenerContainerFactory;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.autoconfigure.http.HttpMessageConverters;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import com.persagy.apm.dmpalarm.json.JacksonMapper;

/**
 * @author: yaoll
 * @date: 2020-09-22
 * @verison: 1.0
 */
@Configuration
public class WebConfig implements WebMvcConfigurer {

	@Bean
	public DmpParameterStorage dmpParameterStorage() {
		return new DmpParameterStorage();
	}

	@Override
	public void addInterceptors(InterceptorRegistry registry) {
		// 设置拦截的路径、不拦截的路径、优先级等等
		registry.addInterceptor(dmpParameterStorage()).addPathPatterns("/**");
	}

	@Bean
	public HttpMessageConverters customConverters() {
		MappingJackson2HttpMessageConverter converter = new MappingJackson2HttpMessageConverter();
		converter.setObjectMapper(JacksonMapper.nonEmptyMapper);
		return new HttpMessageConverters(converter);
	}
	// JMS 使用
	@Bean
	public MessageConverter customMessageConvert() {
		return new Jackson2JsonMessageConverter();
	}

	@Bean
	public RabbitListenerContainerFactory<?> rabbitListenerContainerFactory(ConnectionFactory connectionFactory) {
		SimpleRabbitListenerContainerFactory factory = new SimpleRabbitListenerContainerFactory();
		factory.setConnectionFactory(connectionFactory);
		factory.setMessageConverter(new Jackson2JsonMessageConverter());
		factory.setAcknowledgeMode(AcknowledgeMode.MANUAL);
		return factory;
	}

}
