package com.persagy.apm.dmpalarm.controller;

import java.util.List;

import javax.annotation.Resource;

import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.persagy.apm.dmpalarm.criteria.JsonCriteria;
import com.persagy.apm.dmpalarm.model.AlarmConfig;
import com.persagy.apm.dmpalarm.service.IAlarmConfigService;
import com.persagy.apm.dmpalarm.web.MapResponse;
import com.persagy.apm.dmpalarm.web.PagedResponse;

import io.swagger.annotations.Api;

/**
 * 报警定义控制层
 *
 * @author lixing
 * @version V1.0 2021-12-09 10:43:56
 */
@Api(tags = "报警定义")
@Validated
@RestController("dmpAlarmConfig")
@RequestMapping("/alarm/config")
public class AlarmConfigController {
    @Resource(name="dmpAlarmConfigService")
    private IAlarmConfigService alarmConfigService;

    @PostMapping("/query")
    public PagedResponse<AlarmConfig> query(@RequestBody JsonCriteria criteria) {
        return alarmConfigService.query(criteria);
    }

    @PostMapping("/create")
    public MapResponse create(@RequestBody AlarmConfig alarmConfig) {
        return alarmConfigService.create(alarmConfig);
    }

    /**
     * @description: 批量创建报警定义
     * @param: param
     * @return: com.persagy.common.web.MapResponse
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/22 2:37 下午
     * @version: V1.0
     */
    @PostMapping("/batch_create")
    public MapResponse batchCreate(@RequestBody AlarmConfig alarmConfig) {
        List<AlarmConfig> configModels = alarmConfig.getAlarmConfigs();
        if (configModels == null) {
            MapResponse response = new MapResponse();
            response.setFail("请传入要创建的报警定义！");
            return response;
        }
        return alarmConfigService.batchCreate(configModels);
    }

    @PostMapping("/update")
    public MapResponse update(@RequestBody AlarmConfig param) {
        return alarmConfigService.update(param);
    }

    /**
     * @description: 批量更新报警定义
     * @param: param
     * @return: com.persagy.common.web.MapResponse
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/22 2:37 下午
     * @version: V1.0
     */
    @PostMapping("/batch_update")
    public MapResponse batchUpdate(@RequestBody AlarmConfig alarmConfig) {
        List<AlarmConfig> configModels = alarmConfig.getAlarmConfigs();
        if (configModels == null) {
            MapResponse response = new MapResponse();
            response.setFail("请传入要修改的报警定义！");
            return response;
        }
        return alarmConfigService.batchUpdate(configModels);
    }

    @PostMapping("/delete")
    public MapResponse delete(@RequestBody AlarmConfig alarmConfig) {
        return alarmConfigService.delete(alarmConfig);
    }

    @PostMapping("/batch_delete")
    public MapResponse batchDelete(@RequestBody AlarmConfig alarmConfig) {
        return alarmConfigService.batchDelete(alarmConfig);
    }


    @PostMapping("/init")
    public MapResponse init(@RequestBody AlarmConfig alarmConfig) {
        return alarmConfigService.init(alarmConfig);
    }

}

