package com.persagy.apm.dmpalarm.dao;

import java.util.List;
import java.util.Map;

import org.apache.ibatis.annotations.Delete;
import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Result;
import org.apache.ibatis.annotations.Results;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;

import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.persagy.apm.dmpalarm.model.ShardingEntity;

@Mapper
public interface ShardingMapper {
	/**
	 * 得到数据库分表的所有表名以及对应的count
	 * 
	 * @param tableNamePrefix 表名的前缀
	 * @param startTime       查询的开始时间
	 * @param endTime         查询的结束时间
	 * @return
	 */
	@Select("<script>SELECT TABLE_NAME tableName,suffix FROM ( "
			+ "		SELECT TABLE_NAME, CONVERT(SUBSTRING_INDEX(TABLE_NAME,'_',-1),SIGNED) suffix "
			+ "		FROM information_schema.tables  "
			+ "		WHERE table_schema=(SELECT DATABASE()) AND TABLE_NAME LIKE CONCAT('%',#{tableNamePrefix},'%') "
			+ "		)ls "
			+ "		<where> "
			+ "			<if test=\"null!= startTime and startTime > 0\"> "
			+ "			  AND suffix &gt;=#{startTime}"
			+ "			</if>"
			+ "			<if test=\"null!= endTime and endTime > 0\"> "
			+ "			  AND suffix &lt;=#{endTime}"
			+ "			</if>"
			+ "		</where>"
			+ "		 ORDER BY suffix DESC;</script> ")
	@Results({  
		@Result(column="tableName", property="tableName"),  
		@Result(column="suffix", property="suffix")
		})  
	List<ShardingEntity> getTableNameShardingList(@Param("tableNamePrefix") String tableNamePrefix,
			@Param("startTime") Integer startTime, @Param("endTime") Integer endTime);

	/**
	 * 查询分表后的总记录数
	 * 
	 * @param queryCountSql    执行的SQL
	 * @param newParamValueMap
	 * @return
	 */
	@Select("${queryCountSql}")
	Long selectShardingCount(@Param("queryCountSql") String queryCountSql,
			@Param("ew") Map<String, Object> newParamValueMap);

	/**
	 * 查询每一张表分表后的总数
	 * 
	 * @param queryCountSql    执行的SQL
	 * @param newParamValueMap
	 * @return
	 */
	@Select("${queryCountSql}")
	@Results({  
		@Result(column="tableName", property="tableName"),  
		@Result(column="suffix", property="suffix"),
		@Result(column="totalCount", property="totalCount")
		
		}) 
	List<ShardingEntity> selectShardingGroupTableCount(@Param("queryCountSql") String queryCountSql,
			@Param("ew") Map<String, Object> newParamValueMap);
	/**
	 * 查询数据返回List<Map>
	 * @param sql 执行的SQL
	 * @param newParamValueMap
	 * @return
	 */
	@Select("${sql}")
	List<Map<String, Object>> selectShardingListMap(@Param("sql") String sql, @Param("ew") Map<String, Object> newParamValueMap);
	/**
	 * 新增
	 * @param paramMap
	 */
	@Insert("${sql}")
	void insert(Map<String, Object> paramMap); 
	/**
	 * 根据id修改对象
	 * @param obj
	 * @param paramMap
	 */
	@Update("${sql}")
	void updateById(Map<String, Object> paramMap);
	/**
	 * 根据逐渐id删除
	 * @param ewMap
	 */
	@Delete("${ew.sysSql}")
	void deleteById(@Param(Constants.WRAPPER) Map<String, Object> ewMap);

}
