package com.persagy.apm.dmpalarm.jms;

import java.util.HashSet;
import java.util.Set;

import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import com.persagy.apm.dmpalarm.config.DmpParameterStorage;
import com.persagy.apm.dmpalarm.enumeration.EnumAlarmMessageType;
import com.persagy.apm.dmpalarm.json.JacksonMapper;
import com.persagy.apm.dmpalarm.model.DmpMessage;
import com.persagy.apm.dmpalarm.utils.SpringUtils;

import lombok.extern.slf4j.Slf4j;

/**
 * @author: yaoll
 * @date: 2020-10-22
 * @verison: 1.0
 */
@Slf4j
@Component
public class MessageProcesser {

//	@Value("${persagy.dmp.exchange}")
	private String exchange = "exchange-dmp";

//	@Value("${persagy.dmp.alarm.routingKey}")
	private String dmpAlarmRoutingKey = "routing-dmp-alarm";

	/**
	 * 报警定义的routingKey
	 */
	private static final String alarmConfigRoutingKey = "alarm-config-routing-key";

	@Autowired
	private RabbitTemplate rabbitTemplate;

	public void convertAndSend(DmpMessage message) {
		log.info("向rabbitMQ发送消息，message:" + JacksonMapper.toSimpleJson(message));
		// 如果是报警定义的变化，通过报警定义的routingKey发送，否则用报警公用的routingKey发送
		if (EnumAlarmMessageType.ALARM_CONFIGS_CHANGE.getValue().equals(message.getType())) {
			rabbitTemplate.convertAndSend(exchange, alarmConfigRoutingKey, message);
		} else {
			rabbitTemplate.convertAndSend(exchange, dmpAlarmRoutingKey, message);
		}
	}

	private static final Set<String> beanNames = new HashSet<>();

	static {
//		beanNames.add(EnumAlarmMessageType.ALARM_TARGET_CREATE.getValue());
//		beanNames.add(EnumAlarmMessageType.ALARM_TARGET_UPDATE.getValue());
//		beanNames.add(EnumAlarmMessageType.ALARM_ITEM_UPDATE.getValue());
//		beanNames.add(EnumAlarmMessageType.INSTANCE_OBJ_DELETE.getValue());
//		beanNames.add(EnumAlarmMessageType.ALARM_TARGET_DELETE.getValue());
		beanNames.add(EnumAlarmMessageType.ALARM_CONFIG_BATCH_CREATE.getValue());
//		beanNames.add(EnumAlarmMessageType.ALARM_CONFIG_BATCH_DELETE.getValue());
	}

	public void listen(DmpMessage message) {
		String type = message.getType();
		if (!beanNames.contains(type)) {
			return;
		}

		String executorName = type + "Executor";

		MessageExecutor executor = null;
		try {
			executor = SpringUtils.getBean(executorName, MessageExecutor.class);
		} catch (Exception e) {
			log.error(e.getMessage());
		}
		if (executor == null) {
			log.error("bean for [" + executorName + "] not exists");
		}

		try {
			DmpParameterStorage.ParameterData data = new DmpParameterStorage.ParameterData();
			data.setGroupCode(message.getGroupCode());
			data.setProjectId(message.getProjectId());
			data.setAppId(message.getAppId());

			DmpParameterStorage.set(data);

			executor.execute(message);

		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			DmpParameterStorage.clear();
		}
	}

}
