package com.persagy.apm.dmpalarm.jms.executor;

import com.alibaba.fastjson.JSON;
import com.persagy.apm.dmpalarm.config.DmpParameterStorage;
import com.persagy.apm.dmpalarm.jms.MessageExecutor;
import com.persagy.apm.dmpalarm.json.JacksonMapper;
import com.persagy.apm.dmpalarm.model.AlarmConfig;
import com.persagy.apm.dmpalarm.model.DmpMessage;
import com.persagy.apm.dmpalarm.service.IAlarmConfigService;
import com.persagy.apm.dmpalarm.utils.CheckRequiredParam;
import com.persagy.apm.dmpalarm.web.BaseResponse;
import com.persagy.apm.dmpalarm.web.MapResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 批量创建报警定义
 *
 * @author lixing
 * @version V1.0 2021/10/11 8:00 下午
 */
@Slf4j
@Component
public class AlarmConfigBatchCreateExecutor implements MessageExecutor {
    @Autowired
    private IAlarmConfigService alarmConfigService;

    @Override
    public void execute(DmpMessage message) throws Exception {
        /* 验证消息是否符合要求 */
        CheckRequiredParam.check(message, "exts");

        Map<String, Object> exts = message.getExts();
        String key = "alarmConfigs";
        Object obj = exts.get(key);
        List<AlarmConfig> configModels = new ArrayList<>();
        if (obj instanceof ArrayList<?>) {
            for (Object o : (List<?>) obj) {
                AlarmConfig alarmConfigModel = JacksonMapper.toObject(
                        JSON.toJSONString(o), AlarmConfig.class);
                configModels.add(alarmConfigModel);
            }
        }
        if (CollectionUtils.isEmpty(configModels)) {
            return;
        }
        /* 设置通用请求参数 */
        DmpParameterStorage.ParameterData data = new DmpParameterStorage.ParameterData();
        data.setAppId(message.getAppId());
        data.setGroupCode(message.getGroupCode());
        data.setProjectId(message.getProjectId());
        data.setUserId(message.getUserId());
        DmpParameterStorage.set(data);

        MapResponse response = alarmConfigService.batchCreate(configModels);
        if (BaseResponse.FAIL.equals(response.getResult())) {
            log.error("消费AlarmConfigBatchCreate消息异常，{}", response.getMessage());
        }
    }
}
