package com.persagy.apm.dmpalarm.jms.executor;

import javax.annotation.Resource;

import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import com.persagy.apm.dmpalarm.jms.MessageExecutor;
import com.persagy.apm.dmpalarm.json.JacksonMapper;
import com.persagy.apm.dmpalarm.model.DmpMessage;
import com.persagy.apm.dmpalarm.service.IAlarmConfigService;
import com.persagy.apm.dmpalarm.web.BaseResponse;
import com.persagy.apm.dmpalarm.web.MapResponse;

import lombok.extern.slf4j.Slf4j;

/**
 * @author: lixing
 * @date: 2020-10-23
 * @verison: 1.0
 */
@Slf4j
@Component
public class AlarmItemUpdateExecutor implements MessageExecutor {
	@Resource(name="dmpAlarmConfigService")
	private IAlarmConfigService alarmConfigService;

	@Override
	public void execute(DmpMessage message) {
		log.info("AlarmItemUpdate : {}", JacksonMapper.toSimpleJson(message));

		/* 验证消息是否符合要求 */
		String itemCode = message.getStr1();

		if (StringUtils.isEmpty(itemCode)) {
			log.error("AlarmItemUpdate消息中缺少itemCode信息");
		}

		MapResponse response = alarmConfigService.batchUpdateWhenItemUpdate(itemCode, message.getProjectId(), message.getGroupCode());
		if (BaseResponse.FAIL.equals(response.getResult())) {
			log.error("消费AlarmItemUpdate消息异常，{}", response.getMessage());
		}
	}
}
