package com.persagy.apm.dmpalarm.jms.executor;

import javax.annotation.Resource;

import org.springframework.stereotype.Component;

import com.persagy.apm.dmpalarm.jms.MessageExecutor;
import com.persagy.apm.dmpalarm.json.JacksonMapper;
import com.persagy.apm.dmpalarm.model.DmpMessage;
import com.persagy.apm.dmpalarm.service.IAlarmConfigService;
import com.persagy.apm.dmpalarm.utils.CheckRequiredParam;
import com.persagy.apm.dmpalarm.web.BaseResponse;
import com.persagy.apm.dmpalarm.web.MapResponse;

import lombok.extern.slf4j.Slf4j;

/**
 * @author: lixing
 * @date: 2020-10-23
 * @verison: 1.0
 */
@Slf4j
@Component
public class AlarmTargetDeleteExecutor implements MessageExecutor {
	@Resource(name="dmpAlarmConfigService")
	private IAlarmConfigService alarmConfigService;

	@Override
	public void execute(DmpMessage message) {
		log.info("AlarmTargetDelete : {}", JacksonMapper.toSimpleJson(message));

		/* 验证消息是否符合要求 */
		CheckRequiredParam.check(message, "targetId", "projectId", "groupCode");

		MapResponse response = alarmConfigService.batchDeleteWhenTargetDelete(message.getTargetId(), message.getProjectId(), message.getGroupCode());
		if (BaseResponse.FAIL.equals(response.getResult())) {
			log.error("消费AlarmTargetDelete消息异常，{}", response.getMessage());
		}
	}
}
