package com.persagy.apm.dmpalarm.jms.executor;

import javax.annotation.Resource;

import org.springframework.stereotype.Component;

import com.persagy.apm.dmpalarm.jms.MessageExecutor;
import com.persagy.apm.dmpalarm.json.JacksonMapper;
import com.persagy.apm.dmpalarm.model.DmpMessage;
import com.persagy.apm.dmpalarm.service.IAlarmConfigService;
import com.persagy.apm.dmpalarm.utils.CheckRequiredParam;
import com.persagy.apm.dmpalarm.web.BaseResponse;
import com.persagy.apm.dmpalarm.web.MapResponse;

import lombok.extern.slf4j.Slf4j;

/**
 * @description: 对象类删除
 * @author: lixing
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/12/10 10:53 上午
 * @version: V1.0
 */
@Slf4j
@Component
public class InstanceObjectDeleteExecutor implements MessageExecutor {
	@Resource(name="dmpAlarmConfigService")
	private IAlarmConfigService alarmConfigService;

	@Override
	public void execute(DmpMessage message) {
		log.info("InstanceObjDelete : {}", JacksonMapper.toSimpleJson(message));

		/* 验证消息是否符合要求 */
		CheckRequiredParam.check(message, "targetId", "projectId");

		MapResponse response = alarmConfigService.deleteOnRemoveObj(message.getTargetId(), message.getProjectId());
		if (BaseResponse.FAIL.equals(response.getResult())) {
			log.error("消费InstanceObjDelete消息异常，{}", response.getMessage());
		}
	}
}
