package com.persagy.apm.dmpalarm.service.impl;

import java.util.Date;
import java.util.UUID;

import javax.annotation.Resource;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.dmpalarm.criteria.CriteriaUtils;
import com.persagy.apm.dmpalarm.criteria.JsonCriteria;
import com.persagy.apm.dmpalarm.model.AlarmComment;
import com.persagy.apm.dmpalarm.service.IAlarmCommentService;
import com.persagy.apm.dmpalarm.utils.CheckRequiredParam;
import com.persagy.apm.dmpalarm.web.MapResponse;
import com.persagy.apm.dmpalarm.web.PagedResponse;

import lombok.extern.slf4j.Slf4j;

/**
 * 报警批注(AlarmComment) service层
 *
 * @author lixing
 * @version V1.0 2021-12-09 10:44:19
 */
@Service("dmpAlarmCommentService")
@Slf4j
public class AlarmCommentServiceImpl extends BaseServiceImpl<BaseMapper<AlarmComment>, AlarmComment> implements IAlarmCommentService {
 

    public PagedResponse<AlarmComment> query(JsonCriteria criteria) {
        QueryWrapper<AlarmComment> queryWrapper = criteriaUtils.handleAlarmConditions(criteria, AlarmComment.class);
        try {
            return criteriaUtils.getPagedResponse(criteria, AlarmComment.class, queryWrapper, getBaseMapper());
        } catch (Exception e) {
            log.error("alarmComment query page error JsonCriteria is  {}",criteria.toString(),e);
        }
        return new PagedResponse<>();
    }

    @Transactional
    public MapResponse create(AlarmComment entity) {
        MapResponse response = new MapResponse();
        AlarmParam alarmParam = prepareParam(response);
        if (alarmParam == null) {
            return response;
        }
        // 必填项校验
        String checkResult = CheckRequiredParam.check(entity, "recordId", "content");
        if (!StringUtils.isEmpty(checkResult)) {
            response.setFail(checkResult);
            return response;
        }
        entity.setId(UUID.randomUUID().toString());
        entity.setCreateUser(alarmParam.userId);
        entity.setProjectId(alarmParam.projectId);
        entity.setGroupCode(alarmParam.groupCode);
        entity.setCreateTime(new Date());
        save(entity);
        response.add("id", entity.getId());
        return response;
    }

    @Override
    public MapResponse delete(AlarmComment param) {
        MapResponse response = new MapResponse();
        boolean b=removeById(param.getId());
        response.add("result", b);
        return response;
    }

}
