package com.persagy.apm.dmpalarm.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.persagy.apm.dmpalarm.criteria.JsonCriteria;
import com.persagy.apm.dmpalarm.enumeration.AlarmLogType;
import com.persagy.apm.dmpalarm.model.AlarmLog;
import com.persagy.apm.dmpalarm.service.IAlarmLogService;
import com.persagy.apm.dmpalarm.web.MapResponse;
import com.persagy.apm.dmpalarm.web.PagedResponse;

import lombok.extern.slf4j.Slf4j;

/**
 * 报警日志(AlarmLog) service层
 *
 * @author lixing
 * @version V1.0 2021-12-09 10:44:19
 */
@Slf4j
@Service
public class AlarmLogServiceImpl extends BaseServiceImpl<BaseMapper<AlarmLog>, AlarmLog> implements IAlarmLogService {

    @Override
    @Transactional
    public MapResponse create(AlarmLog entity) {
        MapResponse response = new MapResponse();
        try {
            AlarmParam alarmParam = prepareParam(response);
            if (alarmParam == null) {
                return response;
            }
            entity.setId(UUID.randomUUID().toString());
            entity.setCreateTime(new Date());
            entity.setProjectId(alarmParam.projectId);
            entity.setGroupCode(alarmParam.groupCode);
            entity.setCreateUser(alarmParam.userId);
            save(entity);
            response.add("id", entity.getId());
        } catch (Exception e) {
           log.error("alarmLog create error" ,e);
        }
        return response;
    }

    @Override
    public PagedResponse<AlarmLog> query(JsonCriteria criteria) {
        try {
            QueryWrapper<AlarmLog> queryWrapper = criteriaUtils.handleAlarmConditions(criteria, AlarmLog.class);
            return  criteriaUtils.getPagedResponse(criteria, AlarmLog.class, queryWrapper, baseMapper);
        } catch (Exception e) {
           log.error("alarmLog query page error JsonCriteria is {}",criteria.toString(),e);
        }
        return new PagedResponse<>();
    }
    /**
     * 根据报警记录id批量获取 日志信息集合
     * @param recordIdList  记录id
     * @return map
     */
    @Override
    public Map<String,List<AlarmLog>> getAlarmLogMap(List<String> recordIdList) {
        JsonCriteria criteria=new JsonCriteria();
        criteria.getCriteria().put("recordId",recordIdList);
        QueryWrapper<AlarmLog> queryWrapper = criteriaUtils.handleAlarmConditions(criteria, AlarmLog.class);
        PagedResponse<AlarmLog> resp = criteriaUtils.getPagedResponse(criteria, AlarmLog.class, queryWrapper, getBaseMapper());
        List<AlarmLog> logs = resp.getData();
        if (CollectionUtils.isEmpty(logs)) {
            return new HashMap<>();
        } else {
            Map<String,List<AlarmLog>> alarmLogListMap = new HashMap<>();
            logs.forEach(alarmLog -> {
                if (!alarmLogListMap.containsKey(alarmLog.getRecordId())){
                    alarmLogListMap.put(alarmLog.getRecordId(),new ArrayList<>());
                }
                alarmLogListMap.get(alarmLog.getRecordId()).add(alarmLog);
            });
            return alarmLogListMap;
        }
    }

    /**
     * 获取忽略报警的时间
     * @param recordId
     * @return
     */
    @Override
    public Date getIgnoreAlarmTime(String recordId) {
        // 已忽略
        int after = 1;
        return getCreateTime(recordId, AlarmLogType.TREATMODE, after);
    }

    /**
     * 查询日志创建时间
     * @param recordId 报警记录id
     * @param type 日志了下
     * @param after
     * @return
     */
    private Date getCreateTime(String recordId, String type, int after) {
        JsonCriteria criteria=new JsonCriteria();
        criteria.getCriteria().put("recordId", recordId);
        criteria.getCriteria().put("type", type);
        criteria.getCriteria().put("after", after);
        QueryWrapper<AlarmLog> queryWrapper = criteriaUtils.handleAlarmConditions(criteria, AlarmLog.class);
        PagedResponse<AlarmLog> resp = criteriaUtils.getPagedResponse(criteria, AlarmLog.class, queryWrapper, getBaseMapper());
        List<AlarmLog> logs = resp.getData();
        if (CollectionUtils.isEmpty(logs)) {
            return null;
        } else {
            return logs.get(0).getCreateTime();
        }
    }

}
