package com.persagy.apm.dmpalarm.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Objects;

import org.apache.ibatis.type.TypeHandler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.TableFieldInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfoHelper;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.baomidou.mybatisplus.core.toolkit.ReflectionKit;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.persagy.apm.dmpalarm.dao.ShardingMapper;
import com.persagy.apm.dmpalarm.model.ShardingEntity;
import com.persagy.apm.dmpalarm.service.IShardingService;
import com.persagy.apm.dmpalarm.utils.DatePatternStyle;
import com.persagy.apm.dmpalarm.utils.UtilsTools;
import com.persagy.apm.dmpalarm.web.PagedResponse;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.json.JSONUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class ShardingServiceImpl implements IShardingService {

	private static final String ORDER_BY = "ORDER BY";

	private static final String COUNT_SQL = "SELECT COUNT(*) total";

	private static final String ALL_FIELD_SQL = "SELECT * ";

	public static final String SHARDING_COLUMN = "trigger_time";

	private static final String ASC = "asc";

	private static final String DESC = "desc";

	@Autowired
	private ShardingMapper shardingMapper;

	@Override
	public <T> List<ShardingEntity> getTableNameShardingList(Class<T> cls, Date startTime, Date endTime) {
		Integer startTimeInterger = null;
		Integer endTimeInterger = null;
		if (null != startTime) {
			startTimeInterger = Integer.valueOf(DateUtil.format(startTime, DatePatternStyle.PATTERN_YYYYMM));
		}
		if (null != endTime) {
			endTimeInterger = Integer.valueOf(DateUtil.format(endTime, DatePatternStyle.PATTERN_YYYYMM));
		}
		TableInfo tableInfo = TableInfoHelper.getTableInfo(cls);
		if (CharSequenceUtil.isBlank(tableInfo.getTableName())) {
			log.error("getTableNameShardingList tableNamePrefix is empty");
			return new ArrayList<>();
		}
		return shardingMapper.getTableNameShardingList(tableInfo.getTableName(), startTimeInterger, endTimeInterger);
	}

	@Override
	public <T> Long selectShardingCount(Class<T> cls, QueryWrapper<T> queryWrapper, Date startTime, Date endTime) {
		Map<String, Object> newParamValueMap = new HashMap<>();
		List<ShardingEntity> tableList = this.getTableNameShardingList(cls, null, null);
		String sql = this.shardingSplicingSQL(cls, queryWrapper, newParamValueMap, COUNT_SQL, tableList);
		String queryCountSql = "SELECT SUM(ls.total)  FROM ( " + sql + " )ls";
		return shardingMapper.selectShardingCount(queryCountSql, newParamValueMap);
	}

	@Override
	public <T> List<ShardingEntity> selectShardingGroupTableCount(Class<T> cls, QueryWrapper<T> queryWrapper,
			Date startTime, Date endTime) {
		Map<String, Object> newParamValueMap = new HashMap<>();
		List<ShardingEntity> tableList = this.getTableNameShardingList(cls, startTime, endTime);
		String sql = this.shardingSplicingSQL(cls, queryWrapper, newParamValueMap, COUNT_SQL, tableList);
		String queryCountSql = "SELECT total totalCount,tableName FROM ( " + sql + " ) ls";
		return shardingMapper.selectShardingGroupTableCount(queryCountSql, newParamValueMap);
	}

	private <T> String shardingSplicingSQL(Class<T> cls, QueryWrapper<?> queryWrapper,
			Map<String, Object> newParamValueMap, String sqlPrefix, List<ShardingEntity> tableNameList) {
		List<String> tempQuerySqlList = new ArrayList<>();
		String customSqlSegment = queryWrapper.getCustomSqlSegment();
		String orderBySql = "";
		if (customSqlSegment.contains(ORDER_BY)) {
			orderBySql = customSqlSegment.substring(customSqlSegment.lastIndexOf(ORDER_BY), customSqlSegment.length());
		}
		if (orderBySql.contains(SHARDING_COLUMN)) {
			String[] orderBys = orderBySql.split(",");
			for (String string : orderBys) {
				if(!string.contains(SHARDING_COLUMN)) {
					continue;
				}
				int lastIndex = string.lastIndexOf(",");
				lastIndex=lastIndex == -1?string.length():lastIndex;
				int index = string.indexOf(SHARDING_COLUMN);
				String orderByString = string.substring(index, lastIndex);
				if (orderByString.toUpperCase().contains(ASC.toUpperCase()) || !orderByString.toUpperCase().contains(DESC.toUpperCase())) {
					Collections.reverse(tableNameList);
				}
			}
		}
		Map<String, Object> oldParamValueMap = queryWrapper.getParamNameValuePairs();
		for (int i = 0; i < tableNameList.size(); i++) {
			ShardingEntity shardingEntity = tableNameList.get(i);
			String sqlWhere = new String(customSqlSegment.getBytes()).replace(orderBySql, "");
			for (Map.Entry<String, Object> entry : oldParamValueMap.entrySet()) {
				String oldParamKey = Constants.WRAPPER + ".paramNameValuePairs." + entry.getKey();
				String newParamKey = "table@" + shardingEntity.getTableName() + "@" + entry.getKey();
				sqlWhere = sqlWhere.replace(oldParamKey, Constants.WRAPPER_DOT + newParamKey);
				newParamValueMap.put(newParamKey, entry.getValue());
			}
			tempQuerySqlList.add(sqlPrefix + ",'" + shardingEntity.getTableName()
					+ "' tableName,CONVERT(SUBSTRING_INDEX('" + shardingEntity.getTableName()
					+ "','_',-1),SIGNED) suffix  FROM " + shardingEntity.getTableName() + " " + sqlWhere);
		}
		if (tempQuerySqlList.size() > 1) {
			return CharSequenceUtil.join(" UNION ALL ", tempQuerySqlList);
		} else {
			return tempQuerySqlList.get(0);
		}
	}

	@Override
	public void save(Object obj, Date time) {
		TableInfo tableInfo = TableInfoHelper.getTableInfo(obj.getClass());
		String suffix = DateUtil.format(time, DatePatternStyle.PATTERN_YYYYMM);
		Map<String, Object> paramMap = BeanUtil.beanToMap(obj);
		String insertSql = "INSERT INTO " + tableInfo.getTableName() + "_" + suffix + "  (%s) VALUES (%s)";
		String column = "";
		String value = "";
		if (tableInfo.getIdType() != IdType.AUTO) {
			column += tableInfo.getKeyColumn() + ",";
			value += "#{" + tableInfo.getKeyProperty() + "},";
		}
		for (Entry<String, Object> entry : paramMap.entrySet()) {
			if (null != entry.getValue()) {
				for (TableFieldInfo tableFieldInfo : tableInfo.getFieldList()) {
					String property = tableFieldInfo.getProperty();
					if (entry.getKey().equals(property)) {
						column += tableFieldInfo.getColumn() + ",";
						Class<? extends TypeHandler<?>> typeHandler = tableFieldInfo.getTypeHandler();
						String typeHandlerName = "";
						if (null != typeHandler && CharSequenceUtil.isNotBlank(typeHandler.getName())) {
							typeHandlerName = ",typeHandler=" + typeHandler.getName();
						}
						value += "#{" + property + " " + typeHandlerName + "},";
						break;
					}
				}

			}
		}
		String sql = String.format(insertSql, column, value).replace(",)", ")");
		paramMap.put("sql", sql);
		shardingMapper.insert(paramMap);
	}

	@Override
	public void update(Object obj, Date time) {
		TableInfo tableInfo = TableInfoHelper.getTableInfo(obj.getClass());
		String suffix = DateUtil.format(time, DatePatternStyle.PATTERN_YYYYMM);
		String updateSql = "UPDATE " + tableInfo.getTableName() + "_" + suffix + " SET %s  WHERE %s";
		String column = "";
		Map<String, Object> paramMap = BeanUtil.beanToMap(obj);
		for (Entry<String, Object> entry : paramMap.entrySet()) {
			if (null != entry.getValue()) {
				for (TableFieldInfo tableFieldInfo : tableInfo.getFieldList()) {
					String property = tableFieldInfo.getProperty();
					if (entry.getKey().equals(property)) {
						Class<? extends TypeHandler<?>> typeHandler = tableFieldInfo.getTypeHandler();
						String typeHandlerName = "";
						if (null != typeHandler && CharSequenceUtil.isNotBlank(typeHandler.getName())) {
							typeHandlerName = ",typeHandler=" + typeHandler.getName();
						}
						String value = "#{" + property + " " + typeHandlerName + "},";
						column += tableFieldInfo.getColumn() + "=" + value;
						break;
					}
				}

			}
		}

		String sql = String.format(updateSql, column.substring(0, column.length() - 1),
				tableInfo.getKeyColumn() + "=#{" + tableInfo.getKeyProperty() + "}");
		paramMap.put("sql", sql);
		this.shardingMapper.updateById(paramMap);
	}

	@Override
	public void delete(Object obj) {
		List<ShardingEntity> tableList = this.getTableNameShardingList(obj.getClass(), null, null);
		for (ShardingEntity shardingEntity : tableList) {
			DateTime time = DateUtil.parse(shardingEntity.getSuffix() + "", DatePatternStyle.PATTERN_YYYYMM);
			this.delete(obj, time);
		}
	}

	@Override
	public void delete(Object obj, Date time) {
		String suffix = DateUtil.format(time, DatePatternStyle.PATTERN_YYYYMM);
		TableInfo tableInfo = TableInfoHelper.getTableInfo(obj.getClass());
		String tableName = tableInfo.getTableName() + "_" + suffix;
		Map<String, Object> ewMap = new HashMap<>();
		ewMap.put("sysSql", "DELETE FROM " + tableName + " WHERE  " + tableInfo.getKeyColumn() + "=#{ew."
				+ tableInfo.getKeyColumn() + "}");
		Map<String, Object> param = BeanUtil.beanToMap(obj);
		ewMap.put(tableInfo.getKeyColumn(), param.get(tableInfo.getKeyColumn()));
		this.shardingMapper.deleteById(ewMap);
	}

	@Override
	public void saveOrUpdate(Object obj, Date time) {
		if (null != obj) {
			Class<?> cls = obj.getClass();
			TableInfo tableInfo = TableInfoHelper.getTableInfo(cls);
			Assert.notNull(tableInfo, "error: can not execute. because can not find cache of TableInfo for entity!");
			String keyProperty = tableInfo.getKeyProperty();
			Assert.notEmpty(keyProperty, "error: can not execute. because can not find column for id from entity!");
			Object idVal = ReflectionKit.getMethodValue(cls, obj, tableInfo.getKeyProperty());
			if (StringUtils.checkValNull(idVal) || Objects.isNull(getById(obj.getClass(), (Serializable) idVal))) {
				save(obj, time);
			} else {
				update(obj, time);
			}
		}

	}

	@Override
	public <T> T getById(Class<T> cls, Serializable idVal) {
		if (null == idVal) {
			return null;
		}
		TableInfo tableInfo = TableInfoHelper.getTableInfo(cls);
		List<ShardingEntity> tableNameList = getTableNameShardingList(cls, null, null);
		Map<String, Object> newParamValueMap = new HashMap<>();
		QueryWrapper<?> queryWrapper = new QueryWrapper<>(cls);
		queryWrapper.eq(tableInfo.getKeyColumn(), idVal);
		String sql = this.shardingSplicingSQL(cls, queryWrapper, newParamValueMap, ALL_FIELD_SQL, tableNameList);
		String querySql = "SELECT * FROM ( " + sql + " ) ls";
		List<Map<String, Object>> list = shardingMapper.selectShardingListMap(querySql, newParamValueMap);
		List<T> resultList = handleResultList(cls, list);
		T t = null;
		try {
			t = resultList.get(0);
		} catch (IndexOutOfBoundsException e) {
			log.error("not found {}", idVal, e);
		}
		return t;
	}

	@Override
	public <T> PagedResponse<T> selectShardingPageList(Class<T> cls, QueryWrapper<T> queryWrapper, Integer pageNo,
			Integer pageSize, Date startTime, Date endTime) {
		List<T> resultList = new ArrayList<>();
		pageNo = pageNo < 1 ? 1 : pageNo;
		List<ShardingEntity> tableList = this.selectShardingGroupTableCount(cls, queryWrapper, startTime, endTime);
		/**
		 * 总记录数
		 */
		long count = tableList.stream().mapToLong(ShardingEntity::getTotalCount).sum();
		if (tableList.size() == 1) {
			resultList = queryPage(cls, queryWrapper, pageNo, pageSize, tableList, null);
		} else {
			Long limit = (long) ((pageNo - 1) * pageSize);
			for (int i = 0; i < tableList.size(); i++) {
				ShardingEntity shardingEntity = tableList.get(i);
				if (limit < shardingEntity.getTotalCount()) {
					if ((limit + pageSize) <= shardingEntity.getTotalCount()) {
						resultList.addAll(queryPage(cls, queryWrapper, pageNo, pageSize,
								this.getShardingEntityList(shardingEntity), limit));
						break;
					} else {
						resultList.addAll(queryPage(cls, queryWrapper, pageNo, pageSize,
								this.getShardingEntityList(shardingEntity), limit));
						pageSize = (int) (limit + pageSize - shardingEntity.getTotalCount());
						limit = 0l;
					}
				} else {
					limit = limit - shardingEntity.getTotalCount();
				}
			}
		}
		PagedResponse<T> pagedResponse = new PagedResponse<>();
		pagedResponse.setCount(count);
		pagedResponse.setData(resultList);
		return pagedResponse;
	}

	private List<ShardingEntity> getShardingEntityList(ShardingEntity shardingEntity) {
		List<ShardingEntity> fistShardingEntityList = new ArrayList<>();
		fistShardingEntityList.add(shardingEntity);
		return fistShardingEntityList;
	}

	private <T> List<T> queryPage(Class<T> cls, QueryWrapper<T> queryWrapper, Integer pageNo, Integer pageSize,
			List<ShardingEntity> tableList, Long limit) {
		List<T> resultList;
		Map<String, Object> newParamValueMap = new HashMap<>();
		String sql = this.shardingSplicingSQL(cls, queryWrapper, newParamValueMap, ALL_FIELD_SQL, tableList);
		sql = ALL_FIELD_SQL + " FROM (" + sql + ") ls";
		String customSqlSegment = queryWrapper.getCustomSqlSegment();
		if (customSqlSegment.contains(ORDER_BY)) {
			String orderBySql = customSqlSegment.substring(customSqlSegment.lastIndexOf(ORDER_BY),
					customSqlSegment.length());
			sql += " " + orderBySql;
		}
		sql += " limit " + (limit != null ? limit : (pageNo - 1) * pageSize) + "," + pageSize;
		List<Map<String, Object>> list = shardingMapper.selectShardingListMap(sql, newParamValueMap);
		resultList = handleResultList(cls, list);
		return resultList;
	}

	private <T> List<T> handleResultList(Class<T> cls, List<Map<String, Object>> list) {
		List<T> resultList = new ArrayList<>();
		JSONUtil.toJsonStr(null, null);
		for (Map<String, Object> map : list) {
			Map<String, Object> newMap = new HashMap<>();
			for (Entry<String, Object> entry : map.entrySet()) {
				try {
					JSONObject value = JSON.parseObject(entry.getValue().toString());
					newMap.put(UtilsTools.lineToHump(entry.getKey()), value);
				} catch (Exception e) {
					newMap.put(UtilsTools.lineToHump(entry.getKey()), entry.getValue());
				}
			}
			map.clear();
			T t = JSON.parseObject(JSON.toJSONString(newMap), cls);
			resultList.add(t);
		}
		return resultList;
	}

}
