package com.persagy.apm.energyalarmstarter.alarmdata.feign.client;

import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.apm.energyalarmstarter.alarmdata.constant.RequestUrlConstant;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.AlarmClientConfig;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.AlarmUrlParam;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.DmpResult;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.fallback.AlarmClientFallbackFactory;

import feign.Headers;

/**
 * @description: feign调用数据中台报警接口
 * @author: lixing
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/11/27 10:43 上午
 * @version: V1.0
 */
@FeignClient(name = "dmp-alarm" ,url="http://localhost:${server.port}",fallbackFactory = AlarmClientFallbackFactory.class,
        configuration = AlarmClientConfig.class)
public interface AlarmClient {
    /**
     * @description: 查询报警定义
     * @param: alarmUrlParam
     * @param: jsonObject
     * @return: com.persagy.dmp.starter.alarm.feign.DmpResult<org.springframework.boot.configurationprocessor.json.JSONArray>
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/27 3:41 下午
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.ALARM_CONFIG_QUERY)
    DmpResult<JSONArray> queryAlarmConfig(@SpringQueryMap AlarmUrlParam alarmUrlParam, @RequestBody JSONObject jsonObject) throws Exception;

    /**
     * @description: 创建报警定义
     * @param: alarmUrlParam
     * @param: jsonObject
     * @return: com.persagy.dmp.starter.alarm.feign.DmpResult<org.springframework.boot.configurationprocessor.json.JSONObject>
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/27 3:41 下午
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.ALARM_CONFIG_CREATE)
    DmpResult<JSONObject> createAlarmConfig(@SpringQueryMap AlarmUrlParam alarmUrlParam, @RequestBody JSONObject jsonObject) throws Exception;

    /**
     * @description: 批量创建报警定义
     * @param: alarmUrlParam
     * @param: jsonObject
     * @return: com.persagy.dmp.starter.alarm.feign.DmpResult<org.springframework.boot.configurationprocessor.json.JSONObject>
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/27 3:41 下午
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.ALARM_CONFIG_BATCH_CREATE)
    DmpResult<JSONObject> batchCreateAlarmConfig(@SpringQueryMap AlarmUrlParam alarmUrlParam, @RequestBody JSONObject jsonObject) throws Exception;

    /**
     * @description: 更新报警定义
     * @param: alarmUrlParam
     * @param: jsonObject
     * @return: com.persagy.dmp.starter.alarm.feign.DmpResult<org.springframework.boot.configurationprocessor.json.JSONObject>
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/27 3:41 下午
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.ALARM_CONFIG_UPDATE)
    DmpResult<JSONObject> updateAlarmConfig(@SpringQueryMap AlarmUrlParam alarmUrlParam, @RequestBody JSONObject jsonObject) throws Exception;

    /**
     * @description: 批量更新报警定义
     * @param: alarmUrlParam
     * @param: jsonArrayStr
     * @return: com.persagy.dmp.starter.alarm.feign.DmpResult<com.alibaba.fastjson.JSONObject>
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/4 3:59 下午
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.ALARM_CONFIG_BATCH_UPDATE)
    DmpResult<JSONObject> batchUpdateAlarmConfig(@SpringQueryMap AlarmUrlParam alarmUrlParam, @RequestBody JSONObject jsonObject) throws Exception;

    /**
     * @description: 查询报警记录
     * @param: alarmUrlParam
     * @param: jsonObject
     * @return: com.persagy.dmp.starter.alarm.feign.DmpResult<org.springframework.boot.configurationprocessor.json.JSONArray>
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/27 3:41 下午
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.ALARM_RECORD_QUERY)
    @Headers({"acceptEncoding: gzip","contentType: application/json"})
    DmpResult<JSONArray> queryAlarmRecord(@SpringQueryMap AlarmUrlParam alarmUrlParam, @RequestBody JSONObject jsonObject) throws Exception;

    /**
     * @description: 创建报警记录
     * @param: alarmUrlParam
     * @param: jsonObject
     * @return: com.persagy.dmp.starter.alarm.feign.DmpResult<org.springframework.boot.configurationprocessor.json.JSONObject>
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/27 3:41 下午
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.ALARM_RECORD_CREATE)
    DmpResult<JSONObject> createAlarmRecord(@SpringQueryMap AlarmUrlParam alarmUrlParam, @RequestBody JSONObject jsonObject) throws Exception;

    /**
     * @description: 更新报警记录
     * @param: alarmUrlParam
     * @param: jsonObject
     * @return: com.persagy.dmp.starter.alarm.feign.DmpResult<org.springframework.boot.configurationprocessor.json.JSONObject>
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/27 3:41 下午
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.ALARM_RECORD_UPDATE)
    DmpResult<JSONObject> updateAlarmRecord(@SpringQueryMap AlarmUrlParam alarmUrlParam, @RequestBody JSONObject jsonObject) throws Exception;

    /**
     * @description: 查询报警条目
     * @param: alarmUrlParam
     * @param: jsonObject
     * @return: com.persagy.dmp.starter.alarm.feign.DmpResult<org.springframework.boot.configurationprocessor.json.JSONArray>
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/27 3:41 下午
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.ALARM_ITEM_QUERY)
    DmpResult<JSONArray> queryAlarmItem(@SpringQueryMap AlarmUrlParam alarmUrlParam, @RequestBody JSONObject jsonObject) throws Exception;

    /**
     * @description: 创建报警批注
     * @param: alarmUrlParam
     * @param: jsonObject
     * @return: com.persagy.dmp.starter.alarm.feign.DmpResult<org.springframework.boot.configurationprocessor.json.JSONObject>
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/27 3:42 下午
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.ALARM_COMMENT_CREATE)
    DmpResult<JSONObject> createAlarmComment(@SpringQueryMap AlarmUrlParam alarmUrlParam, @RequestBody JSONObject jsonObject) throws Exception;

    /**
     * @description: 查询报警批注
     * @param: alarmUrlParam
     * @param: jsonObject
     * @return: com.persagy.dmp.starter.alarm.feign.DmpResult<org.springframework.boot.configurationprocessor.json.JSONArray>
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/27 3:42 下午
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.ALARM_COMMENT_QUERY)
    DmpResult<JSONArray> queryAlarmComment(@SpringQueryMap AlarmUrlParam alarmUrlParam, @RequestBody JSONObject jsonObject) throws Exception;

    /**
     * @description: 批量更新报警记录
     * @param urlParamDTO
     * @param jsonObject
     * @return
     * @throws Exception
     */
    @PostMapping(RequestUrlConstant.ALARM_RECORD_BATCH_UPDATE)
    DmpResult<JSONObject> batchUpdateAlarmRecord(@SpringQueryMap AlarmUrlParam alarmUrlParam, @RequestBody JSONObject jsonObject) throws Exception;

    /**
     * @description: 删除报警定义
     * @param urlParamDTO
     * @param jsonObject
     * @return
     * @throws Exception
     */
    @PostMapping(RequestUrlConstant.ALARM_CONFIG_DELETE)
    DmpResult<JSONObject> deleteAlarmConfig(@SpringQueryMap AlarmUrlParam alarmUrlParam, @RequestBody JSONObject jsonObject) throws Exception;

    /**
     * @description: 批量删除报警定义
     * @param urlParamDTO
     * @param jsonObject
     * @return
     * @throws Exception
     */
    @PostMapping(RequestUrlConstant.ALARM_CONFIG_BATCH_DELETE)
    DmpResult<JSONObject> batchDeleteAlarmConfig(@SpringQueryMap AlarmUrlParam alarmUrlParam, @RequestBody JSONObject jsonObject) throws Exception;
}
